﻿;___________________________________________________________________________________________________________
;
; Command line Funtion to delete xdata from a selection set with a wildcard filter for applications names
; Using Entmod method (See CAB post on Swamp http://www.theswamp.org/index.php?topic=3080.msg38638#msg38638)
;___________________________________________________________________________________________________________

(defun C:XdataDelete (/ intCount ssSelections strWCFilter)
 (if (and 
      (setq strWCFilter (strcase (getstring "\nEnter Xdata Application WC Filter Name: ")))
      (princ "\nSelect objects to remove xdata from: ")
      (setq ssSelections (ssget))
     )
  (repeat (setq intCount (sslength ssSelections))
   (setq intCount (1- intCount))
   (xdatadelete (vlax-ename->vla-object (ssname ssSelections intCount)) strWCFilter)
  )  
 )
 (princ)
)

;___________________________________________________________________________________________________________
;
; Funtion to delete xdata from an object with a wildcard filter for applications names
; Using Entmod Method
; LISP Syntax: (xdatadelete obj "TEST#")
;___________________________________________________________________________________________________________

(defun XdataDelete (objSelection
                     strWCFilter
                     /
                     entSelection
                     lstApplictions
                     lstFiltered
                     strApplication
                    )
 (setq entSelection (vlax-vla-object->ename objSelection))
 (if (setq lstApplications (mapcar 'car (xdataget objSelection)))
  (foreach strApplication lstApplications
   (if (wcmatch strApplication strWCFilter)
    (setq lstFiltered (cons strApplication lstFiltered))
   )
  )
 )
 (if lstFiltered
  (apply 'and 
         (mapcar '(lambda (X)
                   (if (assoc -3 (entget entSelection (list X)))
                    (entmod (list (cons -1 entSelection) (list -3 (list X))))
                   )
                  )
                  lstFiltered
         )
  )
 )
)

;___________________________________________________________________________________________________________
;
; Funtion to get xdata from an object and return a list of sublists
; LISP Syntax: (xdataget obj) Returns '(("TEST1" 1 0)("TEST2" 2)("TESTA" "A"))
;___________________________________________________________________________________________________________


(defun XDataGet (objSelection 
                 / 
                 intCount 
                 intDXFCode
                 lstAll 
                 lstDXFCodes 
                 lstDataValues
                 lstSub 
                 safDXFCodes 
                 safDataValues
                )
 (if (and objSelection
         (errortrap (quote (vla-getxdata objSelection "" 'safDXFCodes 'safDataValues) )) 
          safDXFCodes
          safDataValues
         (setq lstDXFCodes   (vlax-safearray->list safDXFCodes)) 
         (setq lstDataValues (mapcar 'variant-value (vlax-safearray->list safDataValues)))  
         (setq intCount 0)
     )
  (foreach intDXFCode lstDXFCodes
   (if (= intDXFCode 1001)
    (if lstSub
     (setq lstAll (cons (reverse lstSub) lstAll)
           lstSub (list (nth intCount lstDataValues))
     )
     (setq lstSub (list (nth intCount lstDataValues)))  
    )
    (setq lstSub (cons (nth intCount lstDataValues) lstSub))   
   )
   (setq intCount (1+ intCount))
  )
 )
 (if lstSub (reverse (cons (reverse lstSub) lstAll)))
)

; General Error Trap.

(defun ErrorTrap (symFunction / objError result)
 (if (vl-catch-all-error-p
      (setq objError (vl-catch-all-apply
                     '(lambda (X)(set X (eval symFunction)))
                      (list 'result)))) 
  nil  
  (if result result 'T)
 )
)


(prin1)